﻿using Dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging.Abstractions;
using Moq;
using Shared;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using API.Controllers;
using Microsoft.Extensions.Logging;
using TestAPI.Extensions;

namespace TestAPI
{
    public class InquiryUnitTest
    {
        private readonly Mock<IInquiryService<InquiryDTO>> inquiryService;
        public InquiryUnitTest()
        {
            inquiryService = new Mock<IInquiryService<InquiryDTO>>();
        }


        [Fact]
        public void GetInquiriesListSuccess()
        {
            var inquiryList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiries(1, 4, 0))
                .Returns(inquiryList);
            var InquiryController = new InquiriesController(inquiryService.Object, new NullLogger<InquiriesController>());

            var inquiriesResult = InquiryController.GetInquiries(1, 4, 0);

            if (inquiriesResult is OkObjectResult okObjectResult)
            {
                var valeur = okObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.Equal(GetInquiriesData().ToString(), valeur.ToString());
                Assert.True(inquiryList.SequenceEqual(valeur as IEnumerable<InquiryDTO>, new InquiryIdEqualityComparer()));

            }

        }

        [Fact]
        public void GetinquiryListFail()
        {
            var inquiriesList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiries(1, 3, 0))
                .Returns(new List<InquiryDTO>());
            var InquiryController = new InquiriesController(inquiryService.Object, new NullLogger<InquiriesController>());

            var inquiriesResult = InquiryController.GetInquiries(2, 3, 0);

            if (inquiriesResult is StatusCodeResult statusCodeResult && statusCodeResult.StatusCode == 204)

            {

                Assert.IsNotType<OkObjectResult>(inquiriesResult);
            }

        }






        [Fact]
        public void GetInquiryIdSuccess()
        {
            var inquiryList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiryById(1))
                .Returns(inquiryList[1]);
            var InquiriesController = new InquiriesController(inquiryService.Object, new NullLogger<InquiriesController>() );

            var inquiryResult = InquiriesController.GetInquiryById(1);
            if (inquiryResult is OkObjectResult okObjectResult)
            {
                InquiryDTO valeur = okObjectResult.Value as InquiryDTO;

                Assert.NotNull(valeur);
                Assert.Equal("titre 2", valeur.Title);
                Assert.Equal("Description", valeur.Description);
                Assert.True(valeur.IsUser);
                Assert.Equal(valeur.GetHashCode(), inquiryList[1].GetHashCode());
                Assert.True(valeur.Equals(inquiryList[1]));
                Assert.False(valeur.Equals(new object()));
                Assert.IsType<InquiryDTO>(valeur);
                Assert.Contains(valeur, inquiryList);
            }

        }

        [Fact]
        public void GetInquiryIdFail()
        {
            var inquiryList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiryById(1))
                .Returns(inquiryList[1]);
            var InquiriesController = new InquiriesController(inquiryService.Object,new NullLogger<InquiriesController>());

            var inquiryResult = InquiriesController.GetInquiryById(100);
            if (inquiryResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<InquiryDTO>(valeur);
                Assert.DoesNotContain(valeur, inquiryList);
            }

        }



        [Fact]
        public void GetInquiryTitleSuccess()
        {
            var inquiryList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiryByTitle("ZUDZU"))
                .Returns(inquiryList[2]);
            var InquiriesController = new InquiriesController(inquiryService.Object,new NullLogger<InquiriesController>());

            var userResult = InquiriesController.GetInquiryByTitle("ZUDZU");
            if (userResult is OkObjectResult okObjectResult)
            {
                InquiryDTO valeur = okObjectResult.Value as InquiryDTO;

                Assert.NotNull(valeur);
                Assert.IsType<InquiryDTO>(valeur);
                Assert.Contains(valeur, inquiryList);
                Assert.Equal("ZUDZU", valeur.Title);
                Assert.Equal("OUHHHH", valeur.Description);
                Assert.Equal(2, valeur.Id);
                Assert.False(valeur.IsUser);
            }

        }


        [Fact]
        public void GetInquiryTitleFail()
        {
            var inquiryList = GetInquiriesData();
            inquiryService.Setup(x => x.GetInquiryByTitle("ZUDZU"))
                .Returns(inquiryList[2]);
            var InquiriesController = new InquiriesController(inquiryService.Object, new NullLogger<InquiriesController>());

            var userResult = InquiriesController.GetInquiryByTitle("GYIIieihhh");

            if (userResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<InquiryDTO>(valeur);
                Assert.DoesNotContain(valeur, inquiryList);
                Assert.False(inquiryList == valeur);
            }
        }










        private List<InquiryDTO> GetInquiriesData()
        {
            List<InquiryDTO> inquiriesData = new List<InquiryDTO>(4)
            {
                new (0,"titre 1","La desc",false),
                new (1,"titre 2","Description",true),
                new (2,"ZUDZU","OUHHHH",false),
                new ("titre premium","Ascendant",true),
            };
            return inquiriesData;
        }


    }
}
